﻿#pragma IndependentModule= OpusLoader
#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma DefaultTab={3,20,4}
#pragma ModuleName=OpusBrowser
#pragma version=1.00
#include <Resize Controls>
#include <WaveSelectorWidget>

// Browse nested keypairs within wavenotes.

// Uses nested keypairs to record acquisition settings for spectroscopic
// data, usually extracted from a data file when data is imported, and
// for parameters that describe subsequent processing of the data.

// Adds items to trace and (for Igor 9+) browser popup menus
#define TraceMenu
#define DBmenu

static strconstant ksPackageName = "OpusBrowser"
static strconstant ksPrefsFileName = "OpusBrowser.bin"
static constant kPrefsVersion = 100

static strconstant ksPanel = "OpusBrowserPanel"
static strconstant ksFilterListbox = "ListBoxWaves"

static structure PackagePrefs
	uint16 version
	uint32 options
	char keysep[2]
	char keylistsep[2]
	char subkeysep[2]
	char subkeylistsep[2]
	STRUCT Rect win // window position and size, 8 bytes
	char reserved[128 - 22]
endstructure

// set prefs structure to default values
static function PrefsSetDefaults(STRUCT PackagePrefs &prefs)
	prefs.version = kPrefsVersion
	prefs.options = 0
	prefs.keysep = ":"
	prefs.keylistsep = "\r"
	prefs.subkeysep = "="
	prefs.subkeylistsep = ";"
	prefs.win.left = 100
	prefs.win.top = 100
	prefs.win.right = prefs.win.left + 600
	prefs.win.bottom = prefs.win.top + 340
	int i
	for(i=0;i<(128-22);i+=1)
		prefs.reserved[i] = 0
	endfor
end

static structure KeyPrefs
	char keysep[2]
	char keylistsep[2]
	char subkeysep[2]
	char subkeylistsep[2]
endstructure

static function LoadPrefs(STRUCT PackagePrefs &prefs)
	LoadPackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
	if (V_flag!=0 || V_bytesRead==0 || prefs.version!=kPrefsVersion)
		PrefsSetDefaults(prefs)
		SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
	endif
end

static function SetOptions()
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	string keysep = prefs.keysep
	string keylistsep = prefs.keylistsep
	string subkeysep = prefs.subkeysep
	string subkeylistsep = prefs.subkeylistsep
//	variable noEditPopNum = 1 + (prefs.options & 1)
	
	Prompt keysep, "Key separator"
	Prompt keylistsep, "List separator"
	Prompt subkeysep, "Subkey separator"
	Prompt subkeylistsep, "Subkey list separator"
//	Prompt noEditPopNum, "Allow editing", Popup, "yes;no;"
	DoPrompt "Opus Browser Settings", Keysep, Keylistsep, Subkeysep, Subkeylistsep//, noEditPopNum
	
	if (V_flag == 1)
		return 0
	endif
	
//	int noEdit = noEditPopNum - 1 // yes = 0, no = 1
	prefs.keysep = keysep
	prefs.keylistsep = keylistsep
	prefs.subkeysep = subkeysep
	prefs.subkeylistsep = subkeylistsep
//	prefs.options = (prefs.options & ~1) | noEdit
	
	SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
//	Button btnDoIt, win=OpusBrowserPanel, disable = prefs.options & 1
	
	ControlUpdate/A/W=OpusBrowserPanel
	return 0
end

// save window position in package prefs
static function SaveWindowPosition(string strWin)
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	// save window position
	GetWindow $strWin wsizeRM
	prefs.win.top    = v_top
	prefs.win.left   = v_left
	prefs.win.bottom = v_bottom
	prefs.win.right  = v_right
	SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
end

menu "Data"
	"Browse OPUS parameters", /Q, OpusBrowser#MakeOpusBrowser()
end

#ifdef TraceMenu
menu "TracePopup"
	"Browse OPUS parameters", /Q, OpusBrowser#DoTraceSelection()
end
#endif

#if IgorVersion() >= 9
#ifdef DBmenu
menu "DataBrowserObjectsPopup"
	"Browse OPUS Parameters", /Q, OpusBrowser#DoDataBrowserSelection()
	"Plot IR spectrum", /Q, OpusBrowser#BrowserPlotIR()
end
#endif
#endif

static function/S ValidBrowserWave(string str, int selectOne, int FPA)
	if (strlen(GetBrowserSelection(-1)) == 0 || (strlen(GetBrowserSelection(1)) && selectOne)  )
		return ""	// Data Browser is not open/multiple objects selected
	endif
	wave/Z w1 = $GetBrowserSelection(0)
	if (WaveExists(w1) && WaveType(w1,1)==1)
		if (FPA==0)
			return str
		elseif (WaveDims(w1)==3)
			return str
		endif
	endif
	return ""
end

static function/S ValidTrace()
	if (WinType("") != 1)
		return "" // don't do anything if Igor is just rebuilding the menu
	endif
	GetLastUserMenuInfo
	if (WinType(S_graphName)==1 && NumberByKey("TYPE", TraceInfo(S_graphName, S_traceName,0))==0)
		return "Browse OPUS Parameters"
	endif
	return ""
end

static function DoTraceSelection()
	GetLastUserMenuInfo
	wave/Z w1 = TraceNameToWaveRef("", S_traceName)
	if (WaveExists(w1))
		MakeOpusBrowser(w=w1)
	endif
end

// browser has already been checked for single wave selection
static function DoDataBrowserSelection()
	wave/Z w1 = $GetBrowserSelection(0)
	if (WaveExists(w1))
		MakeOpusBrowser(w=w1)
	else
		MakeOpusBrowser()
	endif
	return 0
end

static function MakeOpusBrowser([wave/Z w])
	
	KillWindow/Z OpusBrowserPanel
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	DFREF dfr = GetDFR()

	NewPanel/K=1/N=OpusBrowserPanel/W=(prefs.win.left,prefs.win.top,prefs.win.right,prefs.win.bottom) as "OPUS Parameter Browser"
	ModifyPanel/W=OpusBrowserPanel, noEdit=1
	
	InsertWaveSelectorWidget() // adds listbox ksFilterListbox, notebook subwindow and button ButtonClear
	if (WaveExists(w))
		WS_SelectAnObject(ksPanel, ksFilterListbox, GetWavesDataFolder(w,2), OpenFoldersAsNeeded=1)
	else
		WS_OpenAFolderFully(ksPanel, ksFilterListbox, GetDataFolder(1))
	endif
	UpdateListWave()
	
	ListBox lbOPUS,pos={155,10},size={435,300},Proc=OpusBrowser#ListBoxProc
	ListBox lbOPUS,listWave=dfr:ListWave,selWave=dfr:SelWave
	ListBox lbOPUS,widths={1,7,10},userColumnResize=1
		
	SetWindow OpusBrowserPanel hook(hNoteBrowser)=OpusBrowser#hookSavePanelPosition
	
	// panel resizing info
	ListBox lbOPUS,userdata(ResizeControlsInfo)=A"!!,G+!!#;-!!#C>J,hs&z!!#N3Bk1ctB3#!4zzzzzzzzzzzzz!!#o2B4uAeB3#!4z"
	ListBox lbOPUS,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	ListBox lbOPUS,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button buttonClear,userdata(ResizeControlsInfo)=A"!!,Fl!!#BWJ,hlS!!#<(z!!#N3Bk1ctB3#!4zzzzzzzzzzzzz!!#N3Bk1ctB3#!4z"
	Button buttonClear,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button buttonClear,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	ListBox ListBoxWaves,userdata(ResizeControlsInfo)=A"!!,A.!!#;-!!#@k!!#BPz!!#](Aon\"Qzzzzzzzzzzzzzz!!#N3Bk1ctB3#!4z"
	ListBox ListBoxWaves,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	ListBox ListBoxWaves,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
		
	SetWindow kwTopWin,userdata(ResizeControlsInfo)=A"!!*'\"z!!#D&!!#Bdzzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzzzzzzzzz!!!"
	SetWindow kwTopWin,userdata(ResizeControlsGuides)= "gV;fgb;fgt;fgr;"
	SetWindow kwTopWin,userdata(ResizeControlsInfogV)= "NAME:gV;WIN:OpusBrowserPanel;TYPE:User;HORIZONTAL:0;POSITION:150.00;GUIDE1:FL;GUIDE2:FR;RELPOSITION:0.25;"
	SetWindow kwTopWin,userdata(ResizeControlsInfofgb)= "NAME:fgb;WIN:OpusBrowserPanel;TYPE:User;HORIZONTAL:1;POSITION:331.00;GUIDE1:FB;GUIDE2:;RELPOSITION:-9;"
	SetWindow kwTopWin,userdata(ResizeControlsInfofgt)= "NAME:fgt;WIN:OpusBrowserPanel;TYPE:User;HORIZONTAL:1;POSITION:314.00;GUIDE1:FB;GUIDE2:;RELPOSITION:-26;"
	SetWindow kwTopWin,userdata(ResizeControlsInfofgr)= "NAME:fgr;WIN:OpusBrowserPanel;TYPE:User;HORIZONTAL:0;POSITION:130.00;GUIDE1:gV;GUIDE2:;RELPOSITION:-20;"
	
	// resizing panel hook
	SetWindow OpusBrowserPanel hook(ResizeControls)=ResizeControls#ResizeControlsHook

end

static function/DF getDFR()
	DFREF dfr = root:Packages:OpusBrowser
	if (DataFolderRefStatus(dfr) != 1)
		return resetDFR()
	endif
	return dfr
end

static function/DF resetDFR()
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:OpusBrowser
	DFREF dfr=root:Packages:OpusBrowser
	
	Make/O/N=(0,3)/T dfr:listWave /WAVE=listWave
	Make/O/N=(0,3) dfr:selWave /WAVE=selWave
//	Make/O/W/U/N=(3,5,3) dfr:colorWave /WAVE=colorWave
//	colorWave = {{0,0,0},{65535,0,0}}
//	MatrixTranspose colorWave
//	Make/O/N=(0,3,3) dfr:selWave /WAVE=selWave
//	SetDimLabel 2, 1, backColors, selWave // define plane 1 as background colors
	
	// store some prefs for quick retrieval
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	STRUCT KeyPrefs s
	s.keysep = prefs.keysep
	s.keylistsep = prefs.keylistsep
	s.subkeysep = prefs.subkeysep
	s.subkeylistsep = prefs.subkeylistsep
	
	Make/O/N=1 dfr:keyprefs /wave=w
	StructPut s, w
	
	return dfr
end

static function hookSavePanelPosition(STRUCT WMWinHookStruct &s)
	if (s.eventCode == 2)
		SaveWindowPosition(s.winName)
		KillDataFolder/Z root:Packages:OpusBrowser
	endif
	return 0
end

// extracts the keys from a keylist
static function/S ListOfKeys(string keyList, string keySepStr, string listSepStr)
	int numItems = ItemsInList(keyList, listSepStr)
	string strKey = "", strValue = "", strOut = ""
	string strExp = "([^"+ keySepStr + "]+)" + keySepStr + "(.*)"
	string strItem = ""
	int i
	for (i=numItems-1;i>=0;i--)
		strItem = StringFromList(i, keyList, listSepStr)
		SplitString/E=strExp strItem, strKey, strValue
		if (strlen(strKey))
			strOut = AddListItem(strKey, strOut)
		endif
	endfor
	return strOut
end

// returns list of parameter names, in translated format
static function/S ListOfSubkeys(string key, string strNote, STRUCT KeyPrefs &prefs)

	string sublist = StringByKey(key, strNote, prefs.Keysep, prefs.Keylistsep)
	string keyList = ListOfKeys(sublist, prefs.Subkeysep, prefs.Subkeylistsep)

	// translate TLA to human-readable strings
	wave/T w = ParameterTranslationWave(key)
	int i, imax
	imax = ItemsInList(keyList)
	string outlist = "", item = ""
	for (i=ItemsInList(keyList)-1;i>=0;i--)
		item = StringFromList(i, keyList)
		FindValue/RMD=[][0,0]/TEXT=item w
		if (v_row > -1)
			outlist = AddListItem(w[v_row][1], outlist)
		else
			outlist = AddListItem(item, outlist)
		endif
	endfor
	return outlist
end

// update old style wavenotes from older versions of opus loader
static function UpdateWaveNote(wave w)
	string strNote = note(w)
	strNote = ConvertTextEncoding(strNote , 1 , 1 , 2, 1)
	string list = "Acquisition Parameters;FT Parameters;Proc Parameters;Sample Parameters;Absorbance Spectrum Parameters;"
	int i
	for (i=ItemsInList(list)-1;i>=0;i--)
		strNote = ReplaceString(StringFromList(i, list) + "\r", strNote, StringFromList(i, list) + ":")
	endfor
	note/K w, strNote
end

static function/S GetValueByNumber(int keyNum, int subkeyNum, string strNote, STRUCT KeyPrefs &prefs)

	string keys = ListOfKeys(strNote, prefs.Keysep, prefs.Keylistsep)
	string key = StringFromList(keyNum, keys)
	string subkey = ""
	string sublist = StringByKey(key, strNote, prefs.Keysep, prefs.Keylistsep)
	string subkeys = ListOfKeys(sublist, prefs.Subkeysep, prefs.Subkeylistsep)
	subkey = StringFromList(subkeyNum, subkeys) // TLA from wavenote
	return GetValueByName(key, subkey, strNote, prefs)
end

// key and subkey are the wavenote (abbreviated) versions
static function/S GetValueByName(string key, string subkey, string strNote, STRUCT KeyPrefs &prefs)

	string value = ""
	if (strlen(subkey))
		string sublist = StringByKey(key, strNote, prefs.Keysep, prefs.Keylistsep)
		value = StringByKey(subkey, sublist, prefs.Subkeysep, prefs.Subkeylistsep)
		// translate value
		value = TranslateItem(key, subkey, value)
	elseif (strlen(key))
		value = StringByKey(key, strNote, prefs.Keysep, prefs.Keylistsep)
	endif
	value = ReplaceString("\\", value, "\\\\")
	return value
end

static function WS_NotificationProc(string SelectedItem, variable EventCode)	
	switch (eventCode)
		case 3: // double click
		case 4:
		case 5:
			UpdateListWave()
	endswitch
	return 0	
end

static function UpdateListWave()
		
	DFREF dfr = getDFR()
	wave/Z/T listWave = dfr:listWave
	wave/Z selWave = dfr:selWave
	STRUCT KeyPrefs s
	StructGet s dfr:keyprefs
	
	wave/Z w = $StringFromList(0, WS_SelectedObjectsList(ksPanel, ksFilterListbox))
	
	string strNote = ""
	if (WaveExists(w))
		UpdateWaveNote(w) // fix for waves loaded by older version of BrukerLoader
		strNote = note(w)
	endif
		
	Make/free/T/N=(0,3) newListWave
	Make/free/N=(0,3) newSelWave
	SetDimLabel 1, 1, Parameter, newListWave
	SetDimLabel 1, 2, Value, newListWave
	
	string keys, subkeys, key, subkey
	int keyNum, subKeyNum, numKeys, numSubKeys, expand
	int line = -1
	
	keys = ListOfKeys(strNote, s.Keysep, s.Keylistsep)
	numKeys = ItemsInList(keys)
	for (keyNum=0;keyNum<numKeys;keyNum++)
		key = StringFromList(keyNum, keys)
		InsertPoints/M=0 DimSize(newListWave, 0), 1, newListWave, newSelWave
		line ++
		newListWave[line][1] = key
				
		if (cmpstr(key, "History")==0) // special case
			expand = 0
			if (WaveExists(selWave))
				FindValue/TEXT=key/RMD=[][1,1] listWave
				if (v_row > -1)
					expand = (selWave[v_row][0] == 0x50)
				endif
			endif
			
			if (expand == 0)
				newSelWave[line][0] = 0x40
				continue
			endif
			newSelWave[line][0] = 0x50
			string strHist = StringByKey(key, strNote, s.Keysep, s.Keylistsep)
			strHist = ReplaceString("\\", strHist, "\\\\")
			strHist = WrapString(strHist, 100)
			int numHistLines = ItemsInList(strHist, "\r")
			int i
			for (i=0;i<numHistLines;i++)
				InsertPoints/M=0 DimSize(newListWave, 0), 1, newListWave, newSelWave
				line ++
				newListWave[line][2] = StringFromList(i, strHist, "\r")
			endfor
			continue
		endif
		
		subkeys = ListOfSubkeys(key, strNote, s)
		if (strlen(subkeys) == 0)
			newListWave[line][2] = GetValueByNumber(keyNum, 0, strNote, s)
			continue
		endif
		expand = 0
		if (WaveExists(selWave))
			FindValue/TEXT=key/TXOP=4/RMD=[][1,1] listWave // this doesn't work if we have duplicate keys
			// i did it this way so that a parameter list will remain expanded when we switch between waves.
			// could keep this and have another option when fired by listbox activity:
			// step backwards and expand according to selwave
			
			if (v_row > -1)
				expand = (selWave[v_row][0] == 0x50)
			endif
		endif
		
		if (expand == 0)
			newSelWave[line][0] = 0x40
			continue
		endif
		newSelWave[line][0] = 0x50
			
		// add subkey lines
		subKeyNum = 0
		numSubKeys = ItemsInList(subkeys)
		for (subKeyNum=0;subKeyNum<numSubKeys;subKeyNum++)
			InsertPoints/M=0 DimSize(newListWave, 0), 1, newListWave, newSelWave
			line ++
			subkey = StringFromList(subKeyNum, subkeys)
			newListWave[line][1] = "    " + subkey
			newListWave[line][2] = GetValueByNumber(keyNum, subkeyNum, strNote, s)
		endfor
	endfor
		
	Duplicate/O/T newListWave ListWave
	Duplicate/O newSelWave SelWave
end

// break on tab
// else on last space before linelen characters
// else on linelen
static function/S WrapString(string s, int lineLen)
	string sout = ""
	int pos
	s = ReplaceString(num2char(9), s, "\r")
	
	for (;strlen(s)>lineLen;)
		pos = strsearch(s, "\r", 0)
		if (pos < 0 || pos > (linelen-1))
			pos = strsearch(s, " ", lineLen-1, 1)
			pos = pos == -1 ? lineLen-1 : pos
		endif
		sout += RemoveEnding(s[0,pos], "\r") + "\r"
		s = s[pos+1, Inf]
	endfor
	sout += s
	sout = ReplaceString("\r\r\r", sout, "\r\r")
	return RemoveEnding(sout, "\r")
end

static function ListBoxProc(STRUCT WMListboxAction &s)

	switch( s.eventCode )
		case 11: // column resize
			// this is needed when panel AND column widths are resizeable
			ControlInfo /W=$(s.win) $(s.ctrlName)
			variable c1, c2, c3
			sscanf S_columnWidths, "%g,%g,%g", c1, c2, c3
			c1 /= 10; c2 /= 10; c3 /= 10
			ListBox $(s.ctrlName) win=$(s.win), widths={c1, c2, c3}
			break
		case 13: // checkbox clicked (Igor 6.2 or later)
			if (s.eventMod & 2)
				WAVE/Z selWave = s.selWave
				int selection = selWave[s.row][s.col]
				selWave[][0] = selwave == 0 ? 0 : selection
			endif
			UpdateListWave()
			break
		case 1: // mousedown
			if (s.eventMod & 16) // right click?
				RightClick()
			endif
			break
	endswitch

	return 0
end

static function RightClick() // this is in development...
	if (strlen(FunctionInfo("OpusLoader#LoadOpusParameters")) == 0)
		return 0
	endif
	
	return 0

	PopupContextualMenu "Reload from file...;Browse wave;"
	if (v_flag == 1)
		
//		string newParameters = ""
//		#if exists("opusloader#LoadOpusParameters") == 6
//			newParameters = opusloader#LoadOpusParameters()
//		#endif
//		if (strlen(newParameters) == 0)
//			return 0
//		endif
//
//		wave/Z w = $StringFromList(0, WS_SelectedObjectsList(ksPanel, ksFilterListbox))
//		if (WaveExists(w) == 0)
//			return 0
//		endif
//		ClearOpusParameters(w)
//		string strNote = note(w)
//
//		note/K w, newParameters
//		note w, strNote
//		UpdateListWave()
	endif
	
//	if (v_flag == 2)
//		CreateBrowser select=""
//
//	endif
end

//// better to check each line for a match
//function ClearOpusParameters(wave w)
//	string itemList = "Acquisition Parameters;FT Parameters;Proc Parameters;Sample Parameters;Absorbance Spectrum Parameters;History;Spectrum Parameters;Instrument Parameters;IFM Parameters;Optic Parameters;"
//	string strNote = note(w)
//	int numLines = itemsinlist(strNote, "\r"), numItems = itemsinlist(itemList, ";")
//	int i, j
//
//	string line = "", strNewNote = ""
//	for (i=0;i<numLines;i++)
//		line = stringfromlist(i, strNote)
//		for (j=0;j<numItems;j++)
//			if (stringmatch(
//	endfor
//
//
//	note/K w, strNote
//end

// maybe buttons would be useful?
static function OpusButtonProc(STRUCT WMButtonAction &s)
	if (s.eventCode != 2)
		return 0
	endif
	
	strswitch (s.ctrlName)
		case "btnSettings" :
			SetOptions()
			break
		case "btnDone" :
			KillWindow/Z OpusBrowserPanel
			break
	endswitch
	return 0
end

// PNG: width= 90, height= 30
static Picture pCog
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"&!!!!?#R18/!3BT8GQ7^D&TgHDFAm*iFE_/6AH5;7DfQssEc39jTBQ
	=U%adj95u_NKeXCo&'5*oW5;QerS2ddE(FP;'6Ql#oBiuS1dW/'TBu9uWoJd,<?@,Ku[OT_*C6>N&V
	5fgKO-Z!0m]BHZFE$kSY/b+).^UIV80KE-KEW"FQ9_,eF^bT7+UPEE3;1R;IlL+VJZGMtc/)57aYSB
	X^5kLMq7Sp9q0YBG@U214ph]9b)&eA4@(!#Ip%g0/F0_UH>\E'7M.=ui]mI6`m'G"T)B'OA[IVmQDm
	=?Eat`-Qd[5bR1c#F]`cT(Se*XgJen[DI=<Y8YF7N!d-X+iY2pq;SVDt;siWS_bs$!E]WT^pRhs]aH
	M%csq\akmJ4tMo<HGU1WqT,5cib%XrHM`>*]/&U:](RQ7QDj\u&#cMNi8>`?8;-?rCcXX>+1^gW10F
	n!h-2G-M;R\a+TD2op'?`R]"%W!DpJmO],i(@-(/F?,;L7Vkt%YOa,f\8+CV@lHVOEMgMZPnh$6>!V
	MTYBm^8f[O,?Z$2MnH6.T'JWSM4HtSissRo-):4d:ecoe5Tn^(gUEQm+o94@;L(/[91%aXk:!pP;mm
	\kh$s.7qbe%=-p1eBtDs*CHp:+$CUY\0A,jM0:4J2&pY-HWBG?nb`"BE/M-#*)+E?I*C/(r;J]APNh
	3!Ea<(u)`o?0R`ma=QV<n?GV/s3:I0Wf2_M0p@:__T%OEl+sL@10K8&ViQgR(0Q3qMLYA':/iba:,;
	]Y$@ACMV&9b[fD4A`Vq5+A!37VD0na`;0#fWNWKq#f5N>Mt)$S['[2:?=(p2$Q$$NX_cXoJ`iVOcHm
	Rb+"_b#*b4@tp)Xq9r*1_<^IVlpMJ=kE>MhiHa2]]q9<d*4(lA_8$4ej2NM5Z!#`oc=+Ttk-]%D5"O
	Yiu,o$V/I<=@2fN3Ds,PNfIEnqn6C?^[OYDs4q2k*s6TFu+@1>SKUmdko@B5>Pp)-]8`l_Ig,/1c.T
	K'Z+asa)qDc*mqZAKmijlOd;;&H$MEMWY1:\q<G#aaNVlho?TWCGL35!G658MH$RpQ,/[:S#==eP-@
	T+s%'h-7&.0)\eW6j@1gNW'FYlgRIid1g1dP0.MtL)"o@*4A+B&XU\9bRSJWg?B!keI%b6T7FS'?W(
	@7j-a-n[,Adkh%U((6"oN9G"iRV$rmb0"2lqXk08!N&V_b13Oo*tc)h=[L]E@W<ihr:]%Dbs-*cCbc
	T^`<b81D1(d_gue7JX)rMl-Q!ag!0.a4mbCL5'MQH2X<p3`1nA#69QNiWDnN[J^:kIm`JPCXo#W8lo
	?KFN_dOMp#7s\i!;q:1Vb`q^Za5j'0Sg8ALVn\tm:OM*.G/IF;=K4S+O/0U]^`u\O,i'69Y*0'f,SH
	:Kp)mH3.EQ3hDf%?f;W[LbJbR7R5Lb$8U4I7,[8ZM7fU7H5(>6BlSlr1cG6"263q!"YH.\`>aLYN(!
	e3hZYm666$Mq_c(_GHO?%CE(rnI-UV=I6M\e$%CXt$`9q"IB8d`/4e)0&Dcf`43oobf6MqdV?d>\73
	X/dI(2jZ+#[Nt'sQg5KRFl_^rEVX>cZ5h3g1gR#*IhTZf+KqmsVrW[`UcE6>MM*N0fMcT[S!:h'=ju
	EaL@5>OfU]WXd+d/JISLYCQZ_BPkB$IiAYUBq1l^ecC4a8EYJ'WJ,pak5V59k6$F23m\(d</D&W$.c
	&:n64N(\`j0%h;m7iB=j)(R^kh8BUCfn<.JP[26K1F7\>0JOc56jWCOX(6k=i$m^+A%<G5QZ.i$e8]
	01cU/k$R?&@c[1P\L>tK[OkSMmQ7lT_puI&4&#-'R99q+p;3\Sn`Ic2G%kDj/"N6oB<E0?Z6Kl"@,Y
	?4P5G,Nu\q^LTs(Vj(g3YCoZEl;W9!T)>ePG,S!10n_Y\rP(FBr90.J3;jVP^MYp<-ML6>(=)*bFsB
	&uWX-VWg'<P8G@$+\A?J1@G'F,/Z.<q/;.,=8I?;gFp>>;IjEQPE_:7`.R+3bElA@DB6<k@ksJ9lg(
	=CVM=g<G(^E#SiiFHZ8.qF-^ppkE&\[U*_);<'Lfk*Fq]^#W339=Q'IWpj"KgUWU]1,ETW<&G^OtOH
	MkGSX1r0@A!=Gd&>m2;$s(nF8b!K?8R`eZjO8V38_I$<1AcnP!)B*`T#3.X=hE[s8PMoFf*/H2@32u
	N'Len`GUk?n:K*@=9gMMi1ES9gF7flQcCD`2n^,h:`S5=GNQ^G#@^/a:?]W`PV50n4Xue>QVk8E1=]
	lWKB?uV(SiXjL_hVC,G-+W-bHd)[C`^u(BPM:VV58ltJcZ8d$CEhp-Ek)Qb.'^'f)4eT`-]7*:O[1>
	WO?>HRRZ3%&Aum4gNS.jU=.^S*B#H\'0H3Z)tG>O;)WW1_IO09[>@PI4Y3R7J0]^!UgQhj+u1-,O_#
	4q7aj2?Y5.V]ph=D*J`g8?n%JH:8P)K%MLrfVTs(X1]A:d+mFtdNBG"";'8siHNZC4&bKJq`%mNb7r
	SW;=`2-+n=L)HDOsFHoS$CX_6m<3W758iRSt7"9?7u`s%5^"&NsfV]&fKiR),nod"F>!-ZZj257RH'
	BCpkU2>pgE:BYW?=m,Fa:(R[)N0g+8U_d1?h6?87.J?1.S8QM+N.?c/5H^[K9Qq/KSe*09PFi*)kOs
	Cpa8151hB![K\`b9:/2t#`0h)TQGGW^_mOCaj@jCA@uU*q95,uIW@7!X&<O\"P/_=YGg"!\PP(fcMs
	XX8]B01I/5GVm/oHal.qLkA^e?r`*cq5<6cMYpEN]i/%/Vl(p'^fIMd8C\oHLnT1!)7oQm]mKeID*1
	Nn<=:2#ZkEe#YpHHH,[1j)T%7I4;@/)4cu_Q1)PbZM:[@i"UEI%;r>p03XoVZ_RkU=+$'7#=USG/bL
	8-+m<=>h,iqN=A8kNQ3E0-ce+TlUO7L$\:&5CW07\^Y5(=Lpj3bn5fXf]+hD?H]7Wq?&[-c"7hNK0#
	/)B'Mj<HT8ma3CurC,*bh\'aHR7Z[QbX-ZmAk800m):lpO:?P+(!;TY'R\j"AmjWGZ^'4n^bf?W3J@
	>&NBK1HqWkVhSJ@2:%U$9,hq:d,G1cCmI-TcrP\J*Ut[2@6?BcK3XN6]^DH?sm>]m;PWk0+t]M3*pb
	_i5ToaNr1&dko4ib1O7G-^#a3R58IWd+6c;6ULrU<E06&]A7B"H::^+p=jM"Cht@E-\k9W-F%RN7[f
	g9`s&hll-^kfa*7KZfMd!Yc("^(?tb@(G_h8BF>IEA!<RhTlND,!db&r!Y3m>2$M&6dHhJmnR;"(TN
	7k9deNFM:rt_%M:_\bI5MO2h<A0N#R:ZSrC"&ps\iY*%&:=-;@IrX+"G9!l_&sOI?=_'7)$hsk)[Og
	CfZ9V$@mNB]AS#G_>V6^Z_/)$iF?19\*_+U8'Lh!@O$@74\ogtP<K2K,l#,B;0e6NJ*#oS35C1Gqa?
	[/#EMYbdp\'g083t^Hm.OC']=?TCh[+\@'/CD^$lb9k>s0TnKIaqh4fI!&lDq*\4*U*,*??/2AnId;
	.P@%q^Y_gV7L#<Y@CP!Nm,EKn=&i8<r@!PTa5]H_PQ](fBqn;iMUE,Pl5E-<#!*W9G0Gifi6\]*<o4
	FnfrX,WF^_\F$1nF]^fU-bLO!=bm%5lG.k3$IWMqUu'c@l,R*B4I#7$5RGsBAo;S"q!W&r%8C2/"PK
	bsa<4<J73edT0;DW#W4)GM@uFDONL"9R+_N^([T!(k&'aJc*VLHV&^R;!(_#4Z$g7@#[rnEd4b][m6
	"s2Cf6FX#Yth+![-Bc9;DCc35!#ZOe]>R5l%A3s9r*"E3cZ^HAq!@!X3ZLR*.A7oQ8om.\p[kV)RYJ
	YS_VJ,ke(!91A):`eg`H3<A0s%C//=2RBq,pC2:S]*\P@U_Oa*3.T[g!Hf.uI$^Z574:Ig+a&Rh&b)
	g:i!IBPVCY]Y$?Mc-nM/==coe'#A=jP*M;$C2,4.LP+[KAA[:ZiG]VW6ipmf;5gRtUogbYmG#,M=Y+
	23\;kLm>:;.qMlKqn)uClJ![.i',6Vo?VRu"<5C39QHia.rIUXNcq/492/E<t4:q=5j]#0#BT^2C8R
	r=7NOaA&EGCimE'I"(o(b72sE7cRPmWBnj]tHh/;(=(Hs(iTH$*GN/REC6P4"-OQ)1Z*S9OlNXqYG,
	/qFo"%Xt5WcH)DMDo_@'gn2mp\l)\(b!Y0Pa!2n,Nj%-R@Yjn?WT$E#t(FUbj=.R08ON,:0qYL%:/M
	0]<Q1)2)G':0@s*h8ZZ<4MLPu4'A3d&SI6l9j+cCI%0ltDh?G(&13<0N\Q=?t??;p9Q8$59_n3Rli=
	5b`N"1`$#>;[JNr+$!*^fli%qGlHBaGd#r_gndbH0<a<pR0sE5L0:j)I_t)\EH/7Wqt8QJMd<r<&WK
	8J3cuoH9hij#22_bS-?/1q+bUC@(DjDc_1Dg*LCYK([C$_m"OB=44C54XF6CiRHM)#JSik-Qi#lgdX
	C:AAV;n0(-%L_0m!T,"d5MVG@HhTKZ87K.p%WH^Xh3kCpXcTY<@p]%p!H!PcGm8Ma`dWI-i:%O`.@A
	\EMhGlp>Rk7O<G2m`*r,h[u\8;4r,bUaQp%EDN*J]D4B1hFXuppq^tpModARV5%<QlNN?L%hAEkIlW
	/#`^]Bs#-d.f-97RH2#l<o@ZXZ&T?iRH-<%N9SU+'so7AAgW2mil0fWaM)A,6@)4Rp@WFC0@Y,uIN:
	5uCJkMPAJFd6VVd/UR6[rI=?0fVgq/kI,s^(YARDN54WD\PD#"bV<C5XKS<`]Fql#m@)ul]O#Nn]-2
	[lB);<HM,sPARkJs:;d4_S!/nh?qGe7@I:AsnMi7DjM_D$2XW>fsY^ZQI8$;`nm!f"mg^^mq'BNs/!
	!!!j78?7R6=>B
	ASCII85End
end

static function/wave ParameterTranslationWave(string type)
	
	SplitString/E="([^\s]+\s+[^\s]+)" type, type
	Make/free/T/N=(0,2) w
	strswitch (type)
		case "Sample Parameters" :
			w[00][0]= {"PFP","SNM","SFM","CNM","HIS","PTH","EXP","PAT","NAM","NA2","NA3","NA4","ATX","XPP","RAT","CPR","IST","MCP","CPY","DPM","LCT","BLD"}
			w[00][1]= {"Only prompt for","Sample Name","Sample Form","Operator Name","History of Last Operation","Measurement Path","Experiment","Path of File","Filename","Ch. 2 Filename","Ch. 3 Filename"}
			w[11][1]= {"Ch. 4 Filename","Annotation Text","Experiment Path","TRS Method name","Copyright message","Instrument Status","Measurement DLL Parameter","Company","Department","Location","Building"}
			break
		case "Optic Parameters" :
			w[00][0]= {"ACC","SAM","RDX","OBJ","MIC","IRS","UNI","APT","AP2","BMS","DTC","DT2","OF1","OF3","OF4","OPF","OF2","PGN","SST","CHN","RCH","DMX","ADC","SON","SRC","VSC","VEL","HPF","LPF","SRL"}
			w[30][0]= {"RFL","POL","PLA","VAC","CLE"}
			w[00][1]= {"Accessory","Sample String from ID-Chip","Extended Ready Check","Objective","Imaging Device","Iris Aperture (micron)","Command string for UNI","Aperture Setting","Aperture Setting"}
			w[09][1]= {"Beamsplitter Setting","Detector Setting","Detector Setting 2","Optical Filter Setting 1","Optical Filter Setting 3","Optical Filter Setting 4","Optical Filter Setting","Optical Filter Setting 2"}
			w[17][1]= {"Preamplifier Gain","Setpoint Sample Temperature","Measurement Channel","Background Measurement Channel","Multiplexed Data","Ext. Analog Signals","External Synchronisation","Source Setting"}
			w[25][1]= {"Variable Velocity (Hz)","Scanner Velocity","High Pass Filter","Low Pass Filter","Raman Laser Power(mW)","Raman Flags","Polarizer","Polarisation Angle","Vacuum/Flaps","Cleanness Test Threshold"}
			break
		case "Plot and display parameters" :
			w[00][0]= {"PUN","PL0","LFO","PL1","PL2","PL3","PL4","PL5","PL6","PL7","PL8","LGO","PDV","SCP","SCN","PRP","LPP","POP","POF","PLO","PF1","PFI","PLI","PRF","PPA","PWO","PDH","PTX","PLP","PRN"}
			w[30][0]= {"MLW","COS","M00","M01","M02","M03","M04","M05","M06","M07","M08","M09","LJF"}
			w[00][1]= {"units (cm / inch)","Peak Label Size","Peak Label Font","Option Flags","'n' Strongest Peaks to Label","Decimals for Label Numbers","Length Stroke 0 - 1","Length Stroke 1 - 2","Length Stroke 2 - 3"}
			w[09][1]= {"Distance Peak <--> Line","Peak Stroke Length","BRUKER Logo","Plot Device","Script Path","Script Name","Printer Port (LPTn)","Lines per Page","Plot & Print Output Path","Plot & Print Output File Name"}
			w[19][1]= {"Print Log File Name","Plot Option Flags","First Item Printed","Last Item Printed","Print Option Flags","Polyline Parameters","Plot WYSIWYG Options","Plot Dialog Window Handle"}
			w[27][1]= {"Print Text","Print Log Position on Screen","Print Device","Minimum Spectrum Line Width in Plot","Color Settings in Plot","Plot Message # 0","Plot Message # 1","Plot Message # 2"}
			w[35][1]= {"Plot Message # 3","Plot Message # 4","Plot Message # 5","Plot Message # 6","Plot Message # 7","Plot Message # 8","Plot Message # 9","Lab journal file"}
			break
		case "FT Parameters" :
			w[00][0]= {"AF2","HQ2","LQ2","PH2","SP2","ZF2","APF","HFQ","LFQ","PHZ","PHR","NLI","NL2","DIG","DI2","SPZ","ZFF"}
			w[00][1]= {"Apodization Function","End Frequency Limit for File","Start Frequency Limit for File","Phase Correction Mode","Stored Phase Mode","Zero Filling Factor","Apodization Function","End Frequency Limit for File"}
			w[08][1]= {"Start Frequency Limit for File","Phase Correction Mode","Phase Resolution","Non Linearity Correction","Non Linearity Correction","Digital Filter","Digital Filter","Stored Phase Mode"}
			w[16][1]= {"Zero Filling Factor"}
			break
		case "Acquisition Parameters" :
			w[000][0]= {"CH2","SGN","GNS","SG2","SG3","SG4","RGN","RG2","GSW","GSG","AQM","NSS","NSR","REP","DDM","DLR","MIN","MIR","SOS","SOT","STR","COR","CMA","CCO","AQR","DLY","DEL","HFW","LFW"}
			w[029][0]= {"RES","RE2","TDL","ADT","TCL","PLF","SPO","RPO","WAR","WTR","WAS","WRC","WTD","WPD","WXP","WSS","W2W","WXD","WDV","WIR","WIB","WTH","TRR","TRS","TRC","TRD","TRP","TRM","TRX"}
			w[058][0]= {"ITS","ISP","IDL","ITR","STD","STC","SMX","SMU","SMD","SMA","SMF","AMF","ADA","PDA","CIN","CIM","CDT","CDS","CTM","CSV","CTL","GSS","GSO","GSP","CLD","CMD","MLS","MPO","MSH"}
			w[087][0]= {"MDV","MOX","MOY","MDX","MDY","MSX","MSY","MGX","MGY","NDV","NOX","NOY","NSX","NSY","NGX","NGY","MEX","MUN","MPX","MPY","MSS","MCI","MIM","MCM","MEM","MSV","MVM","MRL","MGT"}
			w[116][0]= {"MGS","MGO","MGP","MXS","MXE","MYS","MYE","MDM","MDP","XS2","XE2","YS2","YE2","TVR","TVF","TVP","TVS","TVI","TVA","TVT"}
			w[000][1]= {"Channel 2","Signal Gain, Sample","Signal Gain, Sample","Signal Gain, Sample 2nd Channel","Signal Gain, Sample 3rd Channel","Signal Gain, Sample 4th Channel","Signal Gain, Background"}
			w[007][1]= {"Signal Gain, Background 2nd Channel","Gain Switch Window","Gain Switch Gain","Acquisition Mode","Sample Scans","Background Scans","Repeat Count","Display during Measurement"}
			w[015][1]= {"Delay Between Repeats in Sec.","Sample Meas. Duration in Min.","Background Meas. Duration in Min.","Scantime or Scans","Sample Scans or Time","BG Scans or Time","Correlation Test Mode"}
			w[022][1]= {"Correlation mask","Correlation condition","Acquisition to ram","Stabilization Delay","Delay Before Measurement","Wanted High Frequency Limit","Wanted Low Frequency Limit"}
			w[029][1]= {"Resolution","Resolution Ch.2","To do list","Additional Data Treatment","Command line for Additional Data Treatment","Result Spectrum","Sample Number","Background Number"}
			w[037][1]= {"Tr.Rec. Resolution","Tr.Rec. Resolution","Tr.Rec. Slices","Tr.Rec. Repeat Count","Tr.Rec. trigger Delay in points","Tr.Rec. Stab. Delay after Stepping","Tr.Rec. Trigger Mode"}
			w[044][1]= {"Tr.Rec. Sampling Source","Tr.Rec. Channel 2 Weighting","Tr.Rec. Experiment Delay","Transient Recorder","Tr.Rec.Input Range","Tr.Rec.Input Range 2nd channel","Tr.Rec. Weighting Threshold"}
			w[051][1]= {"TRS Resolution in micro sec","TRS Slices","TRS Repeat Count","TRS Exp Delay in msec","TRS Positioning Delay","TRS Experiment Trigger Mode","TRS Sampling Source","Interleaved Time Slices"}
			w[059][1]= {"Interleave Time Res. \xB5sec","Interleave Trigger Delay \xB5sec","Max. Exp. Trigger Rate Hz","Step Scan Pos. Delay in msec","Step Scan Coadd Count","Multiplexer positions"}
			w[065][1]= {"Multiplexer Option","Modulation (0=OFF 1=MOD 2=MOD-DEMOD 4=OLD PHASE 8=AMPL )","Scanner Modulation Amplitude","Scanner Modulation Frequency","Ampl. Modulation Frequency"}
			w[070][1]= {"Ampl. Demodulation Angle","Phase Demodulation Angle","Chrom Integrate Trace","Chrom Integration Method","Chrom Display Trace","Chrom Display Spectrum","Chrom Start Trigger Mode"}
			w[077][1]= {"Chrom Save Mode","Chrom Trigger Level","Gram Schmidt Size","Gram Schmidt Offset","Gram Schmidt Points","Limit Run Duration","Max Run Duration","Map XY List","Map Port (com 1...n)"}
			w[086][1]= {"Map Shape (1..6)","Map Device","Map Origin X","Map Origin Y","Map Sample Dim. X","Map Sample Dim. Y","Map Spacing X","Map Spacing Y","Map Gram Schmidt X Base","Map Gram Schmidt Y Base"}
			w[096][1]= {"Map Device","Map Origin X","Map Origin Y","Map Spacing X","Map Spacing Y","Map Gram Schmidt X Base","Map Gram Schmidt Y Base","Map Measurement Experiment","Map units","Map # Pos X"}
			w[106][1]= {"Map # Pos Y","Map Save Spectra","Map Compute Integrals","Map Integration Method","Map Macro","Map Evaluation Macro","Map Save Video","Map Video Method","Map Relative Origin"}
			w[115][1]= {"Map Gram Schmidt","Map Gram Schmidt Size","Map Gram Schmidt Offset","Map Gram Schmidt Points","Meas x-Startpoint Display","Meas x-Endpoint Display","Meas y-Minimum Display"}
			w[122][1]= {"Meas y-Maximum Display","Meas Display Mode","Meas Display Product","Meas2 x-Startpoint Display","Meas2 x-Endpoint Display","Meas2 y-Minimum Display","Meas2 y-Maximum Display"}
			w[129][1]= {"Reference Laser Frequency","Fast Scan Frequency","Fast Scan Stage Position","Step Scan Velocity","Step Scan Integration Time","TeraView Parameter String","TeraView scan Type"}
			break
		case "Instrument Parameters" :
			w[000][0]= {"CPQ","CPJ","CPT","CPS","LGN","LOA","LOB","TAX","TAY","IAX","IAY","IBA","TAM","BDP","CPP","CPI","DPH","MOF","NLA","NLB","HFL","LFL","DFR","DFC","HFF","LFF","ASG","AG2","AG3"}
			w[029][0]= {"AG4","ARG","ALF","AHF","ASS","ARS","GFW","GBW","BFW","BBW","DUR","RSN","PKA","PKL","PRA","PRL","P2A","P2L","P2R","P2K","P3A","P3L","P3R","P3K","P4A","P4L","P4R","P4K","DAQ"}
			w[058][0]= {"SSM","SSP","SGP","SGW","TPX","NSN","MVD","INS","ITF","SIM","DEB","LOG","ADR","AD2","RMX","PLL","FFT","MXD","FOC","ABP","NFL","LWN","RLW","RLP","RDY","RCM","RC0","RC1","QC3"}
			w[087][0]= {"QC4","QC5","BSF","BMF","BLT","SRT","ERT","MAX","MAY","AN1","AN2","HUM","TSC","TSM","PIM","PRS","TRG","INT","PCT","AFR","SCA","SSC","CSR","BGT","SRG","CIT","AEX","GRT","CLM"}
			w[116][0]= {"VBI","HBI","BGA","ACL","CTD","CTT","BDT","DTB","LPX","LPW","LPH","TDS","TDE","TDB","GRP","RLT","LNO","LS1","LS2","LS3","LS4","RNO","SC1","SC2","SC3","SC4","TCC","CTS","RHN"}
			w[145][0]= {"SHN","GRN","CLT","CRT","VSN","SRN"}
			w[000][1]= {"Outer motor proportional","Outer motor integral","Mirror tilt proportional","Mirror tilt integral","Laser Gain","Laser Offset laser A","Laser Offset laser B","Target value of Alignment X"}
			w[008][1]= {"Target value of Alignment Y","Initial value of Alignment X","Initial value of Alignment Y","Initial value of laser B Amplitude","True Alignement Mode","BMS Dynamics Parameter"}
			w[014][1]= {"Control Parameter Proportional","Control Parameter Integral","Demod. Phase (Degrees)","Modulation Frequency","NL Alpha","NL Beta","High Folding Limit","Low Folding Limit"}
			w[022][1]= {"Digital Filter Reduction","Number of Filter Coef.","Digital Filter HFL","Digital Filter LFL","Actual Signal Gain","Actual Signal Gain 2nd Channel","Actual Signal Gain 3rd Channel"}
			w[029][1]= {"Actual Signal Gain 4th Channel","Actual Ref. Signal Gain","Actual Low Pass Filter","Actual High Pass Filter","Number of Sample Scans","Number of Background Scans","Number of Good FW Scans"}
			w[036][1]= {"Number of Good BW Scans","Number of Bad FW Scans","Number of Bad BW Scans","Scan time (sec)","Running Sample Number","Peak Amplitude","Peak Location","Backward Peak Amplitude"}
			w[044][1]= {"Backward Peak Location","Peak Amplitude 2nd Channel","Peak Location 2nd Channel","Backward Peak Amplitude 2nd Channel","Backward Peak Location 2nd Channel","Peak Amplitude 3rd Channel"}
			w[050][1]= {"Peak Location 3rd Channel","Backward Peak Amplitude 3rd Channel","Backward Peak Location 3rd Channel","Peak Amplitude 4th Channel","Peak Location 4th Channel","Backward Peak Amplitude 4th Channel"}
			w[056][1]= {"Backward Peak Location 4th Channel","Data Aquisition Status","Sample Spacing Multiplicator","Sample Spacing Divisor","Switch Gain Position","Gain Switch Window","Total Points X"}
			w[063][1]= {"Scan Number","Max. Velocity Deviation","Instrument Type","Interface Type for Optic","Simulation Mode","Debug Printer Mode","Logfile for Measurement","AQP Adress","AQP2 Adress"}
			w[072][1]= {"Resolution Limit","Maximum PLL Setting","Maximum FT Size in K's","Maximum ADC Rate in Hz","Focal Length","Absolute Peak Pos in Laser*2","Nominal FW Peak Pos in Points","Laser Wavenumber"}
			w[080][1]= {"Raman Laser Wavenumber","Raman Laser Power in mW","Ready Check","Ready conditions met","Raman Background Corrected","Raman Scattering Corrected","Calibration Lamp Spectrum Path"}
			w[087][1]= {"Calibration Lamp Spectrum","Temp. of Calibration Lamp","Background Spectrum File","Background Management Flags","Background Lifetime","Start time (sec)","End time (sec)"}
			w[094][1]= {"X Measurement Position","Y Measurement Position","Analog Signal 1","Analog Signal 2","Humidity Interferometer","Scanner Temperature","Sample Temperature","Pressure Interferometer"}
			w[102][1]= {"Pressure Interferometer (hPa)","Trigger Position","Integration Time","Counts proportional to integration time","Automatic Flourescence Rejection","SureCal","Spectral Shape Correction"}
			w[109][1]= {"Cosmic Spike Removal","Background Type","Spectral Range","Camera Integration Time","Automatic Integration Time","Grating","Calibration Mirror","Vertical Binning","Horizontal Binning"}
			w[118][1]= {"Automatic Background","Automatic Calibration","Calibration Delta Time","Calibration Delta Temperature","Background Delta Time","Background Delta Temperature","Laser Position (Pixel)"}
			w[125][1]= {"Laser Peak Width","Laser Peak Height","Track Start Row","Track End Row","Track Binning","Grating Position","Raman Laser Temperatur","Laser Neon Offset","Laser SureCal Coefficient 1"}
			w[134][1]= {"Laser SureCal Coefficient 2","Laser SureCal Coefficient 3","Laser SureCal Coefficient 4","Raman Neon Offset","Raman SureCal Coefficient 1","Raman SureCal Coefficient 2"}
			w[140][1]= {"Raman SureCal Coefficient 3","Raman SureCal Coefficient 4","CCD Temperature","CCD Temperature Set","RSCB Host Name","Spectrograph Host Name","Grating Name","Last Laser Calibration"}
			w[148][1]= {"Last Raman Calibration","Firmware version","Instrument Serial Number"}
			break
		case "Data Parameters" :
            w[00][0]= {"DPF","NPT","FXV","LXV","CSF","MXY","MNY","DXU","DYU","DER","QS1","SMO","QS0","DAT","TIM","XTX","YTX","ZTX","XAF","YAF","ZAF","GSQ","AOX","AOY","AOZ","DDX","DDY","DDZ","NPX","NPY"}
            w[30][0]= {"NPZ","MTR","LAT","EVG","L50","WNO","RMP","RMF","NF1","NF2","SN1","SN2","SN3","SN4","SN5","SN6","SNF","MWC","AWC","HQU","COM","DST","MCC","ALW","RDS","RDR","IRD","LRU"}
            w[00][1]= {"Data Point Format","Number of Data Points","Frequency of First Point","Frequency of Last Point","Y - Scaling Factor","Y - Maximum","Y - Minimum","X Units","Y Units","Derivative"}
            w[10][1]= {"Derivative","Smoothing points for der.","Smoothing points","Date of Measurement","Time of Measurement","X - Axis Label","Y - Axis Label","Z Axis Label","X axis factor","Y axis factor"}
            w[20][1]= {"Z axis factor","GS Base Quality","Map Origin X","Map Origin Y","Map Origin Z","Map Delta X","Map Delta Y","Map Delta Z","Map Points in X","Map Points in Y","Map Points in Z","Maximum Transmission"}
            w[32][1]= {"Layer Thickness","Even Gradient","Lambda 50","Wave Number","Reaction Monitoring Method Path","Reaction Monitoring Method File","First S/N Frequency Limit","Second S/N Frequency Limit"}
            w[40][1]= {"S/N (RMS)","S/N (Peak-to-Peak)","Max. Ordinate in S/N region","Min. Ordinate in S/N region","Noise (RMS)","Noise (Peak-to-Peak)","S/N Flags","Mult. for Freq.Calib","Add for Freq.Calib"}
            w[49][1]= {"Hit Quality","Compound Name","Display text","Correlation conditions met","Actual laser wavenumber","Number of time segments within raw data period","Raw Data Rate in Hz","Delay of IR against laser in raw data periods"}
            w[57][1]= {"Sequence of actions available in wizard"}
			break
		case "Communication Parameters" :
			w[000][0]= {"JAC","JCP","JCC","OEX","ORP","JDT","JDF","COF","COP","REN","SAN","DFN","XCO","YCO","OE1","SOX","QD1","QD2","CBY","DPA","DPO","SEP","ADP","YON","DPL","FQ1","FQ2","FLF","DBT"}
			w[029][0]= {"SII","SIM","SMI","SPM","SID","SAD","CBT","XPF","XST","XPR","XPA","XWI","XWS","XCW","XSB","XEM","XDM","XVP","XPM","DDE","DDS","DDT","DDI","DDD","VBS","VBP","VBW","VBH"}
			w[057][0]= {"PCS","PCP","PCW","PCH","PCB","PLU","PCC","HSN","HSP","HSM","RPL","CMT","EPC","MYN","MYP","MYC","PIN","PIS","PIF","CO1","CO2","RSR","RTX","RRL","RRT","RWT","TIO","CFI"}
			w[085][0]= {"COT","TWK","TPO","TPD","TMP","TCS","TDV","ST1","TD1","TD2","TD3","TD4","ET1","ET2","ET3","ET4","EQ1","EQ2","EQ3","EQ4","RHR","PUD","PTM","SPB","PRM","USN","MDN","MCP"}
			w[113][0]= {"MCD"}
			w[000][1]= {"JCAMP-DX Accept Comma","JCAMP-DX Path","Generate JCAMP-DX Compound Files","Overwrite Existing Files","Ordinate Precision","JCAMP-DX Data Type","JCAMP-DX Date Format","Copy Flags"}
			w[008][1]= {"Copy Path","New File Name","'Save As' File Name","File Name (Data Point Table)","x Column (Data Point Table)","y Column (Data Point Table)","Overwrite (Data Point Table)"}
			w[015][1]= {"Sort X Values (Data Point Table)","Max. Number of Data Points","1st Data Point Line","To Clipboard","Decimal Places, Abscissa","Decimal Places, Ordinate","Separator","All Data Points"}
			w[023][1]= {"y Values only","Use File Limits","X-Startpoint","X-Endpoint","File Load Flags","Data Block Type","User Index in Signature File","Meaning","Index of selected Method Type in Sign Method function  "}
			w[032][1]= {"Protect Mode","MacID","Date","New Block Type","Run as Opus Task","Start Server","Program Name","Parameters","Run in a Window","Window Size Option","Close Window on Exit"}
			w[043][1]= {"Start in Background","Windows Enhanced Mode","VDM Settings Filename","View Transactions","Old or New Function","DDE Interaction","DDE Server Name","DDE Topic","DDE Item"}
			w[052][1]= {"DDE Data","VB Script Name","Script Parameters","Wait for Script","Hide Script","Process Script Name","Process Parameters","Wait for Script","Hide Script","Process Batchnumber"}
			w[062][1]= {"List of last used scenario files","Process Control Code","Converted Script Name","Converted Script Path","Script with Measurement","Replay Line","Comment for History"}
			w[069][1]= {"Comment for Edit Parameter","MyInstrument Script Name","MyInstrument Script Path","MyInstrument Control","Pipe Name","Pipe String","Pipe Flags","COMmunication Port","Mode Line"}
			w[078][1]= {"Read Write Mode","Text to send","Line Separator (Write)","Terminator (Write)","Terminator (Read)  ","Timeout","COMmunication Output File","COMmunication Output Text","Temperature work for thread"}
			w[087][1]= {"Temperature Port (com 1...n)","Use Com Defaults","Temperature","Temperature Command String","Temperature Control Device","Start temperature","Temperature difference 1"}
			w[094][1]= {"Temperature difference 2","Temperature difference 3","Temperature difference 4","End Temperature 1","End Temperature 2","End Temperature 3","End Temperature 4","Equilibrium time 1"}
			w[102][1]= {"Equilibrium time 2","Equilibrium time 3","Equilibrium time 4","Ramp heating rate deg/sec","Ramp Up and Down mode","Temperature Ramp mode","Sample Measurements per Background   "}
			w[109][1]= {"Protein Repeat Measurements               ","Filename uses Sample Name","Measurement DLL NAME","Measurement DLL Parameter","Measurement Command Mode"}
			break
		default :
			w = {{""},{""}}
	endswitch
	return w
end

static function/S TranslateItem(string parameterList, string TLA, string item)
	wave/Z/T w = ItemTranslationWave(parameterList, TLA)
	if (WaveExists(w))
		FindValue/TEXT=item/RMD=[][0,0] w
		if (V_row > -1)
			return w[v_row][1]
		endif
	endif
	return item
end

static function/wave ItemTranslationWave(string parameterList, string TLA)
	
	SplitString/E="([^\s]+\s+[^\s]+)" parameterList, parameterList
	Make/free/T/N=(0,2) w
	strswitch (parameterList)

		case "Data Parameters" :
			strswitch (TLA)
				case "DXU" :
					w[0][0]= {"WN","MI","NM","LGW","MIN","SEC","PNT","EV","MM","CM","MIS","MUS"}
					w[0][1]= {"Wavenumber cm-1","Micron","Nanometers","Log Wavenumber","Minutes","Seconds","Points","eV","Millimeters","Centimeters","msec","\xB5sec"}
		  			return w
		  		case "DYU" : // deprecated
					w[0][0]= {"SC","TR","AB","KM","LA","DR","ABS","REF","TRA","RRK","IRK","RTK","ITK","DF1","DF2","RAM","PAS","EMI","LRF","ATR"}
					w[0][1]= {"Single channel","Transmittance [%]","Absorbance Units","Kubelka Munk","Logarithmic Absorbance","Diffuse Reflectance","Absorbance","Reflectance","Transmittance","Re (Amplitude Reflectivity Coefficient)"}
					w[10][1]= {"Im (Amplitude Reflectivity Coefficient)","Re (Amplitude Transmission Coefficient)","Im (Amplitude Transmission Coefficient)","Re ( Dielectric Function )","Im ( Dielectric Function )","Raman Intensity"}
					w[16][1]= {"Photoacoustic Units","Emission Intensity","Logarithmic Reflectance","ATR Units"}
					return w
  			endswitch
  		
  		case "Instrument Parameters" :
  			strswitch (TLA)
		  		case "LGN" :
					w[0][0]= {"1","2","3","4"}
					w[0][1]= {"1.0","1.8","3.2","5.5"}
					return w
		  		case "TAM" :
					w[0][0]= {"0","1","2"}
					w[0][1]= {"ON","Keep actual position and OFF","Set to Zero and OFF"}
					return w
		  		case "BDP" :
					w[0][0]= {"0","1"}
					w[0][1]= {"Standard","Mylar"}
					return w
		  		case "ITF" :
					w[0][0]= {"0","1","2"}
					w[0][1]= {"25/48","22/28/33/55/66/88/120/100","85/110/113"}
					return w
		  		case "RDY" :
					w[0][0]= {"0","1"}
					w[0][1]= {"OFF","ON"}
					return w
		  		case "RC0" :
		  		case "RC1" :
					w[0][0]= {"NO","YES","BIA"}
					w[0][1]= {"No","Yes","Bias"}
					return w
			endswitch
		
		case "Acquisition Parameters" :
			strswitch (TLA)
				case "CH2" :
					w[0][0]= {"0","1"}
					w[0][1]= {"OFF","ON"}
					return w
		  		case "SGN" :
		  		case "GNS" :
		  		case "SG2" :
		  		case "SG3" :
		  		case "SG4" :
		  		case "RGN" :
		  		case "RG2" :
					w[0][0]= {"-1","0","1","2","3","4","5","6","7"}
					w[0][1]= {"Automatic","1","2","4","8","16","32","64","128"}
					return w
		  		case "GSG" :
					w[0][0]= {"1","8"}
					w[0][1]= {"OFF","ON"}
					return w
		  		case "AQM" :
					w[0][0]= {"SN","DN","SF","SD","DD","DF"}
					w[0][1]= {"Single Sided","Double Sided","Single Sided Fast Return","Single Sided,Forward-Backward","Double Sided,Forward-Backward","Double Sided,Fast Return"}
					return w
		  		case "SOT" :
		  		case "STR" :
					w[0][0]= {"0","1"}
					w[0][1]= {"Scans","Minutes"}
					return w
		  		case "COR" :
					w[0][0]= {"NO","LO","HI","FUL"}
					w[0][1]= {"No","Around Peak,Low","Around Peak,High","Full Igram length"}
					return w
		  		case "PLF" :
					w[0][0]= {"TR","AB","KM","RAM","EMI","RFL","LRF","ATR","PAS"}
					w[0][1]= {"Transmittance","Absorbance","Kubelka Munk","Raman Spectrum","Emission","Reflectance","Log Reflectance","ATR Spectrum","PAS Spectrum"}
					return w
		  		case "SPO" :
		  		case "RPO" :
					w[0][0]= {{"0"},{"Background Position"}}
					return w
		  		case "WXP" :
					w[0][0]= {"1","2","3"}
					w[0][1]= {"Internal","External Positive Edge","External Negative Edge"}
					return w
		  		case "WSS" :
					w[0][0]= {"0","1","2"}
					w[0][1]= {"External","Linear Timescale","Compress to Log. Timescale"}
					return w
		  		case "W2W" :
					w[0][0]= {"-1","0","5","1","6","2","3","4"}
					w[0][1]= {"Not available","Unused","Save","Use for Phase Correction","Use ADC Phase Correction","Use for Weighting","Use for Weighting, discard if < Threshold","Discard Experiment if < Threshold"}
					return w
				case "WDV" :
					w[0][0]= {{"0"},{"Internal ADC"}}
					return w
				case "WIR" :
				case "WIB" :
					w[0][0]= {"10","11","1","2","3","4","5","6","0","7","8","9"}
					w[0][1]= {"\xB150mV","\xB1100mV","\xB1200mV","\xB1500mV","\xB11V","\xB12V","\xB14V","\xB15V","\xB110V","0..400mV","0..1V","0..2V"}
					return w
				case "CSV" :
					w[0][0]= {"0","1","2"}
					w[0][1]= {"Off","On","Auto"}
					return w
				case "MDV" :
					w[0][0]= {"0","1","2","3","4"}
					w[0][1]= {"LANG MCL-2","BRUKER XY","MOT MAP A","MOT MAP B","ScopeII MCL-2"}
					return w
				case "NDV" :
					w[0][0]= {"0","1"}
					w[0][1]= {"Internal (MCLStage)","Microscope (MCLStage)"}
					return w
				case "MUN" :
					w[0][0]= {"0","1","2"}
					w[0][1]= {"Micron","mm","cm"}
					return w
				// don't know how to deal with WTR
			endswitch
		
		case "FT Parameters" :
			strswitch (TLA)
			
				case "AF2" :
				case "APF" :
				case "FTA" :
					w[0][0]= {"BX","TR","4P","HG","B3","B4","NBW","NBM","NBS","US1","US2"}
					w[0][1]= {"Boxcar","Triangular","Four Point","Happ-Genzel","Blackman-Harris 3-term","Blackman-Harris 4-term","Norton-Beer, Weak","Norton-Beer, Medium","Norton-Beer, Strong","User One","User Two"}
					return w
				case "PH2" :
				case "PHZ" :
				case "FHZ" :	// move this to processing
					w[0][0]= {"ML","SM","PW","MLP","SMP","PWP","MS","NO","FM","FS","FP","DP"}
					w[0][1]= {"Mertz","Mertz Signed","Power Spectrum","Mertz / No Peak Search","Mertz Signed / No Peak Search","Power / No Peak Search","Mertz / Stored Phase","No / Save Complex Data","Forman","Forman / Stored Phase"}
					w[10][1]= {"Forman / Preapodized","Doubled Phase"}
					return w
			endswitch
		
		case "Plot and display parameters" :
			strswitch (TLA)
		   		case "PUN" :
  					w[0][0]= {"CM","IN"}
					w[0][1]= {"cm","inch"}
					return w
				case "LGO" :
					w[0][0]= {"NO","YS"}
					w[0][1]= {"No","Yes"}
					return w
			endswitch
						
		case "Optic Parameters" :
			strswitch (TLA)
				case "RDX" :
					w[0][0]= {"0","655"}
					w[0][1]= {"OFF","ON"}
					return w
				case "OBJ" :
					w[0][0]= {"6","1","2","3","4","8","5","13","10","7","9","11","12","14","15","16","17","18"}
					w[0][1]= {"VIS","15 x IR","36 x IR","10 x Raman","40 x Raman","50 x Raman","100 x Raman","EOS Raman","20 x ATR","36 x ATR","Grazing angle","Macro chamber","Macro ch. ATR","HELIOS ATR","20 x Raman","20 x Refl"}
					w[16][1]= {"20 x ATR(Si)","User_defined"}
					return w
				case "MIC" :
					w[0][0]= {{"1"},{"Undefined"}}
					return w
				case "APT" :
					w[0][0]= {"0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15"}
					w[0][1]= {"0.25mm","0.5 mm","1.0 mm","1.5 mm","2.0 mm","2.5 mm","3.0 mm","3.5 mm","4 mm","5 mm","6 mm","7 mm","8 mm","9 mm","10 mm","12 mm"}
					return w
				case "AP2" :
					w[0][0]= {"0","1","2","3","4","5","6","7","8","9","10","11"}
					w[0][1]= {"0.3 mm","0.5 mm","0.65 mm","0.85 mm","1.1 mm","1.4 mm","1.85 mm","2.4 mm","3.5 mm","5.0 mm","7.0 mm","10.0 mm"}
					return w
				case "BMS" :
					w[0][0]= {"-1","-1","-1"}
					w[0][1]= {"KBr","Mylar","Quartz"}
					return w
				case "DTC" :
					w[0][0]= {"0","1","2","3","4","5","7"}
					w[0][1]= {"Test Ramp","DTGS","MCT","Bolometer(extern)","Side Port","Microscope (MCT)","GC-IR (MCT)"}
					return w
				case "DT2" :
					w[0][0]= {"0","1","2","3","4","5","6","7"}
					w[0][1]= {"Test Ramp","MCT D316","InSb","MCT D315","External 4","External 5","External 6","External 7"}
					return w
				case "OF1" :
				case "OF3" :
				case "OF4" :
				case "HPF" : // put this
					w[0][0]= {{"0"},{"OPEN"}}
					return w
				case "OPF" :
				case "OF2" :
					w[0][0]= {"0","1","2","3","4","5","6","7"}
					w[0][1]= {"OPEN","Filter 1","Filter 2","Filter 3","Filter 4","Filter 5","Filter 6","Filter 7"}
					return w
				case "CHN" :
				case "RCH" :
					w[0][0]= {"0","1","3","4","5"}
					w[0][1]= {"Closed","Front","External Beam Side","Microscope","GC-IR / Raman"}
					return w
				case "DMX" :
					w[0][0]= {"1","2","3"}
					w[0][1]= {"ADC 1","ADC 2","ON"}
					return w
				case "ADC" :
					w[0][0]= {"0","1","2","3"}
					w[0][1]= {"OFF","1","2","1 & 2"}
					return w
				case "SON" :
					w[0][0]= {"0","1","2"}
					w[0][1]= {"OFF","ON","Extended"}
					return w
				case "SRC" :
					w[0][0]= {"0","1","1","2"}
					w[0][1]= {"Globar (MIR)","Mercury Lamp (FIR)","Tungsten (NIR)","Emission / Raman"}
					return w
				case "VEL" :
					w[0][0]= {"0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15"}
					w[0][1]= {"0 ;  1.6 KHz","1 ;  2.2 KHz","2 ;  3.0 KHz","3 ;  4.0 KHz","4 ;  5.0 KHz","5 ;  7.5 KHz","6 ; 10.0 KHz","7 ; 20.0 KHz","8 ; 40.0 KHz","9 ; 60.0 KHz","10 ; 80.0 KHz","11 ; 100.0 KHz","12 ; 130.0 KHz"}
					w[13][1]= {"13 ; 140.0 KHz","14 ; 150.0 KHz","15 ; 160.0 KHz"}
					return w
				case "LPF" :
					w[0][0]= {"0","1"}
					w[0][1]= {"Open","1 ; 16 KHz"}
					return w
				case "POL" :
					w[0][0]= {"91","0","90"}
					w[0][1]= {"Out","0\xF8","90\xF8"}
					return w
				case "VAC" :
					w[0][0]= {{"0"},{"Undefined"}}
					return w
			endswitch
			
		case "Processing Parameters" :
			strswitch (TLA)
				case "QAB" :
					w[0][0]= {"SEC","MIN","NM","MI","MM","CM","WN","LGW","EV","PNT","MIS","MUS","NON"}
					w[0][1]= {"Seconds","Minutes","Nanometers","Micrometers","Millimeters","Centimeters","Wavenumber 1/cm","Log Wavenumber","eV","Points","msec","\xB5sec","None"}
					return w
				case "QM0" :
					w[0][0]= {"MI","MM","CM","PNT"}
					w[0][1]= {"Micrometers","Millimeters","Centimeters","Points"}
					return w
				case "QAR" :
					w[0][0]= {"MPP","MI"}
					w[0][1]= {"Micrometers/Pixel","Micrometers"}
					return w
				case "QAX" :
					w[0][0]= {"RCR","ACR","ACT"}
					w[0][1]= {"Recognize Crystal","Accessory Recognition","Accessory Test"}
					return w
				case "FHZ" :
					w[0][0]= {"ML","SM","PW","MS","NO","FM","FS","FP","DP"}
					w[0][1]= {"Mertz","Mertz Signed","Power Spectrum","Mertz / Stored Phase","No / Save Complex Data","Forman","Forman / Stored Phase","Forman / Preapodized","Doubled Phase"}
					return w
				case "FTA" :
					w[0][0]= {"BX","TR","4P","HG","B3","B4","NBW","NBM","NBS","US1","US2"}
					w[0][1]= {"Boxcar","Triangular","Four point","Happ-Genzel","Blackman-Harris 3-Term","Blackman-Harris 4-Term","Norton-Beer, Weak","Norton-Beer, Medium","Norton-Beer, Strong","User One","User Two"}
					return w
				case "FSM" :
					w[0][0]= {"AL","MX","MN","MI","NO","MA","TW","TP"}
					w[0][1]= {"Absolute largest Value","Maximum","Minimum","Mid between Min/Max","No Peak Search","Manually","Mid between largest two","Take from stored Phase"}
					return w
				case "PPF" :
					w[0][0]= {"TR","AB"}
					w[0][1]= {"Transmittance","Absorbance"}
					return w
				case "DSP" :
					w[0][0]= {"LO","GA"}
					w[0][1]= {"Lorentz","Gauss"}
					return w
				case "2DM" :
					w[0][0]= {"SQ","RG","CO","SY","AS","PA"}
					w[0][1]= {"Squared Correlation","Regression","Correlation","Synchron","Asynchron","Phase Angle"}
					return w
				case "QI1" :
					w[0][0]= {"SNM","ID","FIL"}
					w[0][1]= {"Sample Name","ID","File Name"}
					return w
				case "QI5" :
					w[0][0]= {"LTS","WLT","WGS","WSE"}
					w[0][1]= {"Library Tree as Shown","Whole Library Tree","Whole Library Tree and Group Statistics","Whole Library Tree and Statistics for Each Spectrum"}
					return w
				case "QI6" :
					w[0][0]= {"A4","LET"}
					w[0][1]= {"A4","Letter"}
					return w
				case "QI7" :
					w[0][0]= {"POR","LAN"}
					w[0][1]= {"Portrait","Landscape"}
					return w
				case "QCJ" :
					w[0][0]= {"NO","DEN","DIA","BOT"}
					w[0][1]= {"No","Dendrogram","Diagnosis","Both"}
					return w
				case "QCK" :
					w[0][0]= {"FIL","SNM","NUM","NO"}
					w[0][1]= {"File Name","Sample Name","File Number","No Name Markers"}
					return w
				case "NOI" :
					w[0][0]= {"0","1"}
					w[0][1]= {"Yes","No"}
					return w
				case "QPA" :
					w[0][0]= {"NO","YES","OVR","ANY"}
					w[0][1]= {"No","Yes","Overwrite","Anyway"}
					return w
				case "QAL" :
					w[0][0]= {"LIS","FIL"}
					w[0][1]= {"File List","File Name"}
					return w
				case "XAU" :
					w[0][0]= {"NO","YS"}
					w[0][1]= {"Linear","Compressed"}
					return w
				case "SCQ" :
					w[0][0]= {"ABS","REF","TRA","RRK","RTK"}
					w[0][1]= {"Absorbance","Reflectance","Transmittance","Amplitude Reflectivity Coefficient","Amplitude Transmission Coefficient"}
					return w
				case "SPT" :
					w[0][0]= {"HOM","IHO"}
					w[0][1]= {"Homogenious","Inhomogenious"}
					return w
				case "QAU" :
				case "QAV" :
				case "QCQ" :
				case "QCR" :
				case "QI4" :
				case "QQP" :
				case "QQS" :
				case "QP6" :
				case "QP4" :
				case "QP8" :
				case "QP0" :
				case "QPC" :
				case "QAE" :
				case "QAF" :
				case "QC9" :
				case "QCD" :
				case "QCE" :
				case "QF0" :
				case "QT0" :
				case "QT1" :
					w[0][0]= {"NO","YES"}
					w[0][1]= {"No","Yes"}
					return w
			endswitch
			
		case "Communication Parameters" :
			strswitch (TLA)
				case "ORP" :
					w[0][0]= {"32","16"}
					w[0][1]= {"32 Bit","16 Bit"}
					return w
				case "JDT" :
					w[0][0]= {"PAC","SQD","DD"}
					w[0][1]= {"Packed","Squeezed/Dup","Difference/Dup"}
					return w
				case "JDF" :
					w[0][0]= {"DMY","YMD"}
					w[0][1]= {"DD/MM/YY","YY/MM/DD"}
					return w
				case "TDV" :
					w[0][0]= {"0","1","2","3","4","5","6","7","8"}
					w[0][1]= {"Eurotherm 800 Series","Lake Shore 320","Linkam TP93 TP94 Series","Eurotherm 2000 Series","Haake DC 30/K20","Huber Polystat cc3","Haake DC 5","Haake DC 50 internal sensor","Haake DC 50 external sensor"}
					return w
				case "PTM" :
					w[0][0]= {"TF","TR1","TR2","TR3","TR4","CTR"}
					w[0][1]= {"Fixed Temperature","Temperature Ramp (1 Temp-Range)","Temperature Ramp (2 Temp-Ranges)","Temperature Ramp (3 Temp-Ranges)","Temperature Ramp (4 Temp-Ranges)","Constant Heating/Cooling Rate"}
					return w
				case "JCC" :
				case "OEX" :
				case "OE1" :
				case "SOX" :
				case "CBY" :
				case "ADP" :
				case "YON" :
				case "DPL" :
					w[0][0]= {"1","0"}
					w[0][1]= {"Yes","No"}
					return w
			endswitch					
	endswitch
	
	return $""
end

// *** wave selector widget with filter ***
// start of code that adds real-time filtering to wave selector widget
// need to set up guides, search and replace module name.

static function InsertWaveSelectorWidget()
	// wave selector widget with quick filter
	int isWin = 0
	#ifdef WINDOWS
	isWin = 1 // this will be used to fix the positioning of controls for Windows
	#endif
	
	// insert a notebook subwindow to be used for filtering lists
	DefineGuide/W=$ksPanel gV={FL,0.25,FR}
	DefineGuide/W=$ksPanel fgb={FB,-9}, fgt={FB,-26}, fgr={gV,-20}
	NewNotebook/F=1/N=nbFilter/HOST=$ksPanel/W=(10,500,5200,1000)/FG=($"",fgt,fgr,fgb)/OPTS=3
	Notebook $ksPanel+"#nbFilter" fSize=12-3*isWin, showRuler=0
	Notebook $ksPanel+"#nbFilter" spacing={1, 0, 0}
	Notebook $ksPanel+"#nbFilter" margins={0,0,1000}
	SetWindow $ksPanel+"#nbFilter" activeChildFrame=0
	
	DoUpdate/W=$ksPanel
	SetActiveSubwindow $ksPanel
	
	// make a Button for clearing text in notebook subwindow
	Button ButtonClear, win=$ksPanel, pos={135,315}, size={15,15}, title=""
	Button ButtonClear, win=$ksPanel, Picture=OpusBrowser#ClearTextPicture, Proc=OpusBrowser#filterButtonProc, disable=1
	
	ClearText(1) // sets notebook to its default appearance
	
	// insert a listbox for wave selector widget
	// filter will apply to this listbox
	ListBox $ksFilterListbox, win=$ksPanel, pos={10,10}, size={135,300}, focusRing=0
	MakeListIntoWaveSelector(ksPanel, ksFilterListbox, content=WMWS_Waves, selectionMode=WMWS_SelectionSingle, nameFilterProc="OpusBrowser#FilterForWaveSelector")
	WS_SetNotificationProc(ksPanel, ksFilterListbox, "OpusBrowser#WS_NotificationProc")
	WS_OpenAFolderFully(ksPanel, ksFilterListbox, GetDataFolder(1))
		
	SetWindow $ksPanel hook(hFilterNB)=OpusBrowser#hookFilterNB	
end

// function used by WaveSelectorWidget
static function FilterForWaveSelector(string aName, variable contents)

	string nb = ksPanel + "#nbFilter"
	string strFilter = GetUserData(nb, "", "filter")
	
	string leafName = ParseFilePath(0, aName, ":", 1, 0)
	int vFilter = 0
	try
		vFilter = GrepString(leafName, "(?i)" + strFilter); AbortOnRTE
	catch
		int err = GetRTError(1)
	endtry
	return vFilter
end

static function ClearText(int doIt)
	if (doIt) // clear filter widget
		
//		
		
		string nb = ksPanel + "#nbFilter"
		Notebook $nb selection={startOfFile,endofFile}, textRGB=(50000,50000,50000), text="Filter"
		Notebook $nb selection={startOfFile,startOfFile}
		Button buttonClear, win=$ksPanel, disable=3
		SetWindow $nb UserData(filter) = ""
		
	endif
end

// intercept and deal with keyboard events in notebook subwindow
static function hookFilterNB(STRUCT WMWinHookStruct &s)
	
	if (s.eventcode == 4) // mousemoved
		return 0
	endif
	
	string nb = ksPanel + "#nbFilter"
		
	GetWindow/Z $ksPanel activeSW
	if (cmpstr(s_value, nb))
		return 0
	endif
	
	if (s.eventCode == 22 && cmpstr(s.WinName, nb)==0) // don't allow scrolling
		return 1
	endif
	
	string strFilter = GetUserData(nb, "", "filter")
	int vLength = strlen(strFilter)
	
	if (s.eventcode==3 && vLength==0) // mousedown
		return 1 // don't allow mousedown when we have 'filter' displayed in nb
	endif
		
	if (s.eventcode == 10) // menu
		strswitch (s.menuItem)
			case "Paste":
				string strScrap = GetScrapText()
				strScrap = ReplaceString("\r", strScrap, "")
				strScrap = ReplaceString("\n", strScrap, "")
				strScrap = ReplaceString("\t", strScrap, "")
				
				GetSelection Notebook, $nb, 1 // get current position in notebook
				if(vLength == 0)
					Notebook $nb selection={startOfFile,endofFile}, text=strScrap
				else
					Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text=strScrap
				endif
				vLength += strlen(strScrap) - abs(V_endPos - V_startPos)
				s.eventcode = 11
				// pretend this was a keyboard event to allow execution to continue
				break
			case "Cut":
				GetSelection Notebook, $nb, 3 // get current position in notebook
				PutScrapText s_selection
				Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text=""
				vLength -= strlen(s_selection)
				s.eventcode = 11
				break
			case "Clear":
				GetSelection Notebook, $nb, 3 // get current position in notebook
				Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text="" // clear text
				vLength -= strlen(s_selection)
				s.eventcode = 11
				break
		endswitch
		Button buttonClear, win=$ksPanel, disable=3*(vLength == 0)
		ClearText((vLength == 0))
	endif
				
	if (s.eventcode!=11)
		return 0
	endif
	
	if (vLength == 0) // Remove "Filter" text before starting to deal with keyboard activity
		Notebook $nb selection={startOfFile,endofFile}, text=""
	endif
	
	// deal with some non-printing characters
	switch(s.keycode)
		case 9:	// tab: jump to end
		case 3:
		case 13: // enter or return: jump to end
			Notebook $nb selection={endOfFile,endofFile}
			break
		case 28: // left arrow
		case 29: // right arrow
			ClearText((vLength == 0)); return (vLength == 0)
		case 8:
		case 127: // delete or forward delete
			GetSelection Notebook, $nb, 1
			if(V_startPos == V_endPos)
				V_startPos -= (s.keycode == 8)
				V_endPos += (s.keycode == 127)
			endif
			V_startPos = min(vLength,V_startPos); V_endPos = min(vLength,V_endPos)
			V_startPos = max(0, V_startPos); V_endPos = max(0, V_endPos)
			Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text=""
			vLength -= abs(V_endPos - V_startPos)
			break
	endswitch
		
	// find and save current position
	GetSelection Notebook, $nb, 1
	int selEnd = V_endPos
		
	if(strlen(s.keyText) == 1) // a one-byte printing character
		// insert character into current selection
		Notebook $nb text=s.keyText, textRGB=(0,0,0)
		vLength += 1 - abs(V_endPos - V_startPos)
		// find out where we want to leave cursor
		GetSelection Notebook, $nb, 1
		selEnd = V_endPos
	endif
	
	// select and format text
	Notebook $nb selection={startOfFile,endOfFile}, textRGB=(0,0,0)
	// put text into global filter string
	GetSelection Notebook, $nb, 3
	strFilter = s_selection
	Notebook $nb selection={(0,selEnd),(0,selEnd)}, findText={"",1}
	
	Button buttonClear, win=$ksPanel, disable=3*(vLength==0)
	ClearText((vLength == 0))
		
	SetWindow $nb UserData(filter) = strFilter
	
	int savlen = strlen(WS_SelectedObjectsList(ksPanel, ksFilterListbox))
	WS_UpdateWaveSelectorWidget(ksPanel, ksFilterListbox)
	
	// specific to this procedure
	// we may have deselected a wave
	if (strlen(WS_SelectedObjectsList(ksPanel, ksFilterListbox))!=savlen)
		UpdateListWave()
	endif
		
	return 1 // tell Igor we've handled all keyboard events
end

static function FilterButtonProc(STRUCT WMButtonAction &s)
	if(s.eventCode!=2)
		return 0
	endif
	ClearText(1)
	WS_UpdateWaveSelectorWidget(ksPanel, ksFilterListbox)
	return 0
end

// PNG: width= 90, height= 30
static Picture ClearTextPicture
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"&!!!!?#R18/!3BT8GQ7^D&TgHDFAm*iFE_/6AH5;7DfQssEc39jTBQ
	=U"$&q@5u_NKm@(_/W^%DU?TFO*%Pm('G1+?)0-OWfgsSqYDhC]>ST`Z)0"D)K8@Ncp@>C,GnA#([A
	Jb0q`hu`4_P;#bpi`?T]j@medQ0%eKjbh8pO.^'LcCD,L*6P)3#odh%+r"J\$n:)LVlTrTIOm/oL'r
	#E&ce=k6Fiu8aXm1/:;hm?p#L^qI6J;D8?ZBMB_D14&ANkg9GMLR*Xs"/?@4VWUdJ,1MBB0[bECn33
	KZ1__A<"/u9(o<Sf@<$^stNom5GmA@5SIJ$^\D=(p8G;&!HNh)6lgYLfW6>#jE3aT_'W?L>Xr73'A#
	m<7:2<I)2%%Jk$'i-7@>Ml+rPk4?-&B7ph6*MjH9&DV+Uo=D(4f6)f(Z9"SdCXSlj^V?0?8][X1#pG
	[0-Dbk^rVg[Rc^PBH/8U_8QFCWdi&3#DT?k^_gU>S_]F^9g'7.>5F'hcYV%X?$[g4KPRF0=NW^$Z(L
	G'1aoAKLpqS!ei0kB29iHZJgJ(_`LbUX%/C@J6!+"mVIs6V)A,gbdt$K*g_X+Um(2\?XI=m'*tR%i"
	,kQIh51]UETI+HBA)DV:t/sl4<N*#^^=N.<B%00/$P>lNVlic"'Jc$p^ou^SLA\BS?`$Jla/$38;!#
	Q+K;T6T_?\3*d?$+27Ri'PYY-u]-gEMR^<d.ElNUY$#A@tX-ULd\IU&bfX]^T)a;<u7HgR!i2]GBpt
	SiZ1;JHl$jf3!k*jJlX$(ZroR:&!&8D[<-`g,)N96+6gSFVi$$Gr%h:1ioG%bZgmgbcp;2_&rF/[l"
	Qr^V@O-"j&UsEk)HgI'9`W31Wh"3^O,KrI/W'chm_@T!!^1"Y*Hknod`FiW&N@PIluYQdKILa)RK=W
	Ub4(Y)ao_5hG\K-+^73&AnBNQ+'D,6!KY/`F@6)`,V<qS#*-t?,F98]@h"8Y7Kj.%``Q=h4.L(m=Nd
	,%6Vs`ptRkJNBdbpk]$\>hR4"[5SF8$^:q=W([+TB`,%?4h7'ET[Y6F!KJ3fH"9BpILuUI#GoI.rl(
	_DAn[OiS_GkcL7QT`\p%Sos;F.W#_'g]3!!!!j78?7R6=>B
	ASCII85End
end

// end of filter code

static function BrowserPlotIR()
	Display
	int i = 0
	string strWave
	
	Preferences/Q 1
	int savPref = V_flag
	
	do
		strWave = GetBrowserSelection(i)
		wave/Z w1 = $strWave
		if (WaveExists(w1) && waveType(w1, 1)==1 && WaveDims(w1)==1)
			AppendToGraph w1
		endif
		i ++
	while (strlen(strWave))
	Label/Z left "Absorbance"
	Label/Z bottom "Wavenumber (cm\\S-1\\M)"
	ModifyGraph/Z mode=0
	SetAxis/Z/A/R bottom
	Preferences/Q savPref
	return 0
end